/***************************************************************************
 *   Copyright (C) 2007 by Jan Dzik, Alexios Kotsifakos, Harambos Nikolaou,*
 *   Kastrisios Konstantinos, Giorgos Pletsis, Kandias Miltiadis           *
 *   iam.jss@gmail.com   						   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SCANNERRQ_H
#define SCANNERRQ_H

/* system includes */
#include <stdlib.h>
#include <math.h>
#include <curl/curl.h>
#include <pthread.h>

/* project includes */
#include "../scannerDB/scannerdb.h"
#include "../lib/msg_digest.h"


#define METHOD_LENGTH 20
#define URL_LENGTH 128

/* contains information about an answered http request */ 
typedef struct requestInfo {
	long httpStatusCode;		/* the status code of the request as returned by the server */
	char *httpExpectedResponse;	/* the expected response of the request */
	char *httpResponse;		/* the status code of the request as returned by the server 
					   concatenated with the reason phrase */
	char *method;			/* the method applied for the request */
	char *description;		/* the description of the potential vulnerability to this request */
	char *address;
	char md5sum[MSG_DIGEST_LENGTH]; /* the md5 sum of the body of the url */
} requestInfo_t;



/* contains information needed per thread */
typedef struct scannerRQ {
	scannerdb_t *db;	/* sqlite3 connection */
	CURL *curl;		/* */
	int offset;		/* the offset from the start of the buffer to start reading requests */
	int nreqs;		/* how many should it read */
	int counter;		/* how many has it read until now */
	FILE *outputBody;
} scannerRQ_t;

char url[URL_LENGTH];		/* the server to connect and check for vulnerabilities */
scannerRQ_t *threadInfo;	/* the table in which information is stored for every thread */
int threadInfoSize;		/* the size of that table */
scannerdb_t *db;		/* the shared connection to the sqlite3 database */
char *statusLine;		/* a temporary global variable for storing the httpErrorCode and the reason phrase */
int md5Calculation;		/* 1 to calculate md5 sum on the body of the answer by the server */
pthread_mutex_t statusLine_mutex = PTHREAD_MUTEX_INITIALIZER;
				/* race condition mutex for statusLine temporary buffer */

/*
 *  All functions update the variable RQ_Error appropriately. In case that something
 *  goes wrong just call function RQ_PrintError(RQ_Error).
 *  You are the only one responsible for the good course of the program, so you
 *  have the obligation to check whether a call of the RQ_ level succeeded or failed
 *  in order to continue further on.
 */




/* Initializes all structures needed. It has to be called once
 * by the main thread before the creation of the threads
 * @nthreads:	the total number of the threads
 * @url     :	the url to check for vulnerabilities (e.g protocol://domain)
 * @md5sum  :	1 for md5 calculation on the body of the server's answer
 * @return  :	RQ_SCANNER_OK if all went good, or an error code
 * */
int RQ_Init(int nthreads, char *url, int md5sum);



/* Cleans up the space that allocated dynamically by RQ_Init().
 * @return:	RQ_SCANNER_OK if all went good, or an error code
 * */
int RQ_Cleanup(void);



/* Opens a connection to the sqlite3 database.
 * This function must not be called manually.
 * It is called automatically by the RQ_Init() function.
 * @return:	RQ_SCANNER_OK if all went good, or an error code
 * */
int RQ_Open(void);



/* Closes a connection to the sqlite3 database.
 * This function must not be called manually.
 * It is called automatically by the RQ_Cleanup() function.
 * @return:     RQ_SCANNER_OK if all went good, or an error code
 * */
int RQ_Close(void);



/* Makes a new request.
 * @threadId:	the id of the thread make the request 
 * @return  :	NULL if something goes wrong or a pointer
 * 		to a dynamically allocated item requestInfo_t
 *
 * You are under the obligation to keep a copy of this item if you
 * would like to use it at a different time, because a next call to 
 * this function frees the previously allocated memory.
 * */
int RQ_SendNextRequest(int threadId, requestInfo_t *curReq);

/*
 * Necessary function to parse the headers of the server's answer
 * A full description of the arguments can be found at the manpage
 * of curl_easy_setopt under CURLOPT_HEADERFUNCTION.
 * */
size_t getHeader(void *ptr, size_t size, size_t  nmemb,  void  *stream);

int RQ_getURLBody(requestInfo_t *curReq, const char *url);

#endif
